/*
 * Copyright (c) 2005
 *	Stefan Sperling <stsp@binarchy.net>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Stefan Sperling AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Stefan Sperling OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <SDL.h>

#include "wurmer.h"
#include "event.h"
#include "worm.h"
#include "gameboard.h"
#include "graphics.h"
#include "menu.h"
#include "options.h"

static int handle_key(wurmer *w, SDLKey key);
static int handle_menu_command(wurmer *w);
static int handle_menu_parameter_change(wurmer *w, SDLKey key);

/* returns new game state */
int get_input(wurmer *w) {
	SDL_Event event;
	SDL_PollEvent(&event);
	switch(event.type) {
		case SDL_KEYDOWN:
			return handle_key(w, event.key.keysym.sym);
		case SDL_QUIT:
			return GAME_QUIT;
	}

	return w->game_state;
}

static int handle_key(wurmer *w, SDLKey key) {

	if (w->game_state == GAME_RUNNING) {

		if (key == SDLK_ESCAPE) {
			return GAME_MENU;
		}

		int x;
		for (x = 0; x < w->gb->num_worms; x++) {
			if (key == w->gb->worms[x]->leftkey) {
				reorientate(w->gb->worms[x], WORM_GO_LEFT);
				return GAME_RUNNING;
			} else if (key == w->gb->worms[x]->rightkey) {
				reorientate(w->gb->worms[x], WORM_GO_RIGHT);
				return GAME_RUNNING;
			}
		}
	}
	
	else if (w->game_state == GAME_MENU) {
		switch (key) {
			case SDLK_UP:
				if (w->m->active_entry > 0)
					w->m->active_entry--;
				return GAME_MENU;
			case SDLK_DOWN:
				if (w->m->active_entry < NUM_MENU_ENTRIES - 1)
					w->m->active_entry++;
				return GAME_MENU;
			case SDLK_LEFT: /* fall through */
			case SDLK_RIGHT:
				return handle_menu_parameter_change(w, key);
			case SDLK_ESCAPE:
				return GAME_RUNNING;
			case SDLK_RETURN:
				return handle_menu_command(w);
			default:
				return GAME_MENU;
		}
	}
	
	return w->game_state;
}

static int handle_menu_command(wurmer *w) {
	switch (w->m->active_entry) {
		case ENTRY_NEW_GAME:
			reset_gameboard(w);
			return GAME_RUNNING;
		case ENTRY_QUIT_GAME:
			return GAME_QUIT;
		default:
			return w->game_state;
	}
}

static int handle_menu_parameter_change(wurmer *w, SDLKey key) {
	switch (w->m->active_entry) {
		case ENTRY_PLAYERS:
			if (key == SDLK_LEFT) {
				if (w->options->num_worms > MIN_WORMS)
					w->options->num_worms--;
			} else if (key == SDLK_RIGHT) {
				if (w->options->num_worms < MAX_WORMS)
					w->options->num_worms++;
			}
			return w->game_state;

		case ENTRY_RESOLUTION:
			if (key == SDLK_LEFT) {
				if (w->options->gm > 0) {
					w->options->gm--;
					return GAME_NEW;
				}
			} else if (key == SDLK_RIGHT) {
				if (w->options->gm < NUM_GRAPHICS_MODES - 1) {
					w->options->gm++;
					return GAME_NEW;
				}
			}
			break;

		default:
			break;
	}

	return w->game_state;
}

