/*
 * Copyright (c) 2005
 *	Stefan Sperling <stsp@binarchy.net>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Stefan Sperling AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Stefan Sperling OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <SDL.h>

#include "gameboard.h"
#include "graphics.h"
#include "worm.h"
#include "wurmer.h"

char default_worm_names[MAX_WORMS][WORM_NAME_LENGTH] = {
	"stsp\0\0\0\0\0",
	"ole\0\0\0\0\0\0",
	"neels\0\0\0\0",
	"ulli\0\0\0\0\0",
	"jq\0\0\0\0\0\0\0",
	"jeff\0\0\0\0\0",
	"jan\0\0\0\0\0\0",
	"elaine\0\0\0",
};


static gameboard gameboard_templates[NUM_GRAPHICS_MODES] = {
	/* 640x480 */ {
		160,  /* width  */
		120,  /* height */
		4,    /* cell_size */
		1,    /* border_width */
		NULL, /* colors */
		NULL, /* grid */
		MIN_WORMS,    /* num_worms */
		{NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL},  /* worms */
		0,		/* winner */
		0,		/* num_crashed_worms */
		0		/* game_over */
	},
	/* 800x600 */ {
		200,  /* width  */
		150,  /* height */
		4,    /* cell_size */
		1,    /* border_width */
		NULL, /* colors */
		NULL, /* grid */
		MIN_WORMS,    /* num_worms */
		{NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL},  /* worms */
		0,		/* winner */
		0,		/* num_crashed_worms */
		0		/* game_over */
	},
	/* 1024x768 */ {
		256,  /* width  */
		192,  /* height */
		4,    /* cell_size */
		1,    /* border_width */
		NULL, /* colors */
		NULL, /* grid */
		MIN_WORMS,    /* num_worms */
		{NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL},  /* worms */
		0,	 	/* winner */
		0,		/* num_crashed_worms */
		0		/* game_over */
	}
	/* maybe add more sizes in the future */
};

/* Each worm has a unique relative starting position:
 *         0     .25      .75     1   (x)
 *               w4       w6      
 * 0       ------------------------
 *         |                      |
 * .25  w0 |                      | w3
 *         |                      |
 *         |                      |
 * .75  w2 |                      | w1
 *         |                      |
 * 1       ------------------------
 *               w7       w5
 *(y)
 *
 * */
typedef struct worm_start_coordinates {
	float x;
	float y;
	int x_offset;
	int y_offset;
	worm_orientation orientation;
} worm_start_coordinates;

static const worm_start_coordinates start_coordinates[MAX_WORMS] = {
	/* w0 */ {0, 0.25, 1, 0, WORM_LOOKING_EAST},
	/* w1 */ {1, 0.75, -2, 0, WORM_LOOKING_WEST},
	/* w2 */ {0, 0.75, 1, 0, WORM_LOOKING_EAST},
	/* w3 */ {1, 0.25, -2, 0, WORM_LOOKING_WEST},
	/* w4 */ {0.25, 0, 0, 1, WORM_LOOKING_SOUTH},
	/* w5 */ {0.75, 1, 0, -2, WORM_LOOKING_NORTH},
	/* w6 */ {0.75, 0, 0, 1, WORM_LOOKING_SOUTH},
	/* w7 */ {0.25, 1, 0, -2, WORM_LOOKING_NORTH}
};

/* defines default colors for the various gameboard cell states */
SDL_Color gameboard_colors[NUM_CELL_STATES] = {
	/* worm 0 */    {0xff, 0, 0, 0},
	/* worm 1 */ 	{0, 0xff, 0, 0},
	/* worm 2 */ 	{0x33, 0x33, 0xff, 0},
	/* worm 3 */ 	{0xff, 0xff, 0, 0},
	/* worm 4 */ 	{0xff, 0, 0xff, 0},
	/* worm 5 */ 	{0, 0xff, 0xff, 0},
	/* worm 6 */ 	{0xcc, 0, 0, 0},
	/* worm 7 */ 	{0xcc, 0xcc, 0xcc, 0},
	/* CELL_FREE */ {0, 0x22, 0x66, 0},
	/* CELL_WALL */ {0xff, 0xff, 0xff, 0}
};

const SDLKey worm_keys[MAX_WORMS][2] = {
#define KEYBOARD_US /* only us keyboard supported yet */
#ifdef KEYBOARD_US
	/* w0 */ {SDLK_q, SDLK_w},
	/* w1 */ {SDLK_PERIOD, SDLK_SLASH},
	/* w2 */ {SDLK_LESS, SDLK_z},
	/* w3 */ {SDLK_LEFTBRACKET, SDLK_RIGHTBRACKET},
	/* w4 */ {SDLK_r, SDLK_t},
	/* w5 */ {SDLK_n, SDLK_m},
	/* w6 */ {SDLK_u, SDLK_i},
	/* w7 */ {SDLK_c, SDLK_v},
#endif /* KEYBOARD_US */
#undef KEYBOARD_US
};

static worm_crash detect_crash(gameboard_cell cell);

gameboard* new_gameboard(graphics_mode gm, unsigned int num_worms) {

	if (num_worms < MIN_WORMS || num_worms > MAX_WORMS) {
		fprintf(stderr, "%i worms are not supported, using %i\n",
				num_worms, MAX_WORMS);
		num_worms = MAX_WORMS;
	}
	
	if (gm < 0 || gm >= NUM_GRAPHICS_MODES) {
		fprintf(stderr, "Unsupported graphics mode %i\n", gm);
		return NULL;
	}

	gameboard* gb = &gameboard_templates[gm];

	gb->grid = (gameboard_cell*)malloc(sizeof(gameboard_cell) * gb->height
			* gb->width);
	if (gb->grid == NULL) {
		fprintf(stderr, "Could not allocate memory for "
				"gameboard grid\n");
		gb = NULL;
		return NULL;
	}

	/* clear the grid */
	unsigned int x, y;
	for (x = 0; x < gb->width; x++)
		for (y = 0; y < gb->height; y++)
			set_gameboard_cell(gb, x, y, CELL_FREE);

	/* mark the walls */
	for (x = 0; x < gb->width; x++) // top row
		set_gameboard_cell(gb, x, 0, CELL_WALL);
	for (x = 0; x < gb->width; x++) // bottom row
		set_gameboard_cell(gb, x, gb->height - 1, CELL_WALL);
	for (y = 0; y < gb->height; y++) // leftmost row
		set_gameboard_cell(gb, 0, y, CELL_WALL);
	for (y = 0; y < gb->height; y++) // rightmost row
		set_gameboard_cell(gb, gb->width - 1, y, CELL_WALL);

	gb->colors = gameboard_colors;
	gb->num_worms = num_worms;


	/* create worms */
	unsigned int startx, starty;
	for (x = 0; x < num_worms; x++) {
		startx = start_coordinates[x].x * gb->width
			+ start_coordinates[x].x_offset;
		starty = start_coordinates[x].y * gb->height
			+ start_coordinates[x].y_offset;
		gb->worms[x] = new_worm(default_worm_names[x],
				startx, starty,
				start_coordinates[x].orientation,
				&gameboard_colors[x],
				worm_keys[x][0], worm_keys[x][1]);
		if (gb->worms[x] == NULL) {
			fprintf(stderr, "Could not allocate memory for worm "
					"%i\n", x);
			goto new_worm_failed;
		}

		gb->worms[x]->cells_per_step = 1;
	}

	gb->winner = NO_WINNER_YET;
	gb->num_crashed_worms = 0;
	gb->game_over = 0;

	return gb;

new_worm_failed:
	for (x = 0; x < num_worms; x++) {
		if (gb->worms[x] == NULL)
			break;
		else
			kill_worm(gb->worms[x]);
	}
	free(gb->grid);
	gb->grid = NULL;
	gb->colors = NULL;
	gb = NULL;
	return NULL;
}

void kill_gameboard(gameboard *gb) {
	if (gb == NULL)
		return;
	int x;
	for (x = 0; x < gb->num_worms; x++)
		kill_worm(gb->worms[x]);
	if (gb->grid != NULL)
		free(gb->grid);
	gb->grid = NULL;
	gb->colors = NULL;
}


/* move all worms cells_per_step steps forward, if possible.
 * else set crash flag on a worm */
void move_worms(gameboard *gb) {
	worm *w = NULL;
	unsigned int num_worm;
	gameboard_cell cell;

	if (gb->game_over)
		return;

	for (num_worm = 0; num_worm < gb->num_worms; num_worm++) {

		w = gb->worms[num_worm];
		if (w->has_crashed != WORM_NO_CRASH)
			continue;

		unsigned int cells;
#define worm_steps(new_xpos, new_ypos, step) \
	for (cells = 0; cells < w->cells_per_step; cells++) { \
		set_gameboard_cell(gb, w->xpos, w->ypos, num_worm); \
		cell = get_gameboard_cell(gb, new_xpos, new_ypos); \
		w->has_crashed = detect_crash(cell); \
		if (w->has_crashed == WORM_NO_CRASH) \
			step; \
	}
		switch (w->orientation) {
			case WORM_LOOKING_NORTH:
				worm_steps(w->xpos, w->ypos - 1, w->ypos--);
				break;
			case WORM_LOOKING_SOUTH:
				worm_steps(w->xpos, w->ypos + 1, w->ypos++);
				break;
			case WORM_LOOKING_EAST:
				worm_steps(w->xpos + 1, w->ypos, w->xpos++);
				break;
			case WORM_LOOKING_WEST:
				worm_steps(w->xpos - 1, w->ypos, w->xpos--);
				break;
			default:
				fprintf(stderr, "Unknown orientation %i "
						"for worm %s\n", w->orientation,
						w->name);
				return;
		}

		if (w->has_crashed != WORM_NO_CRASH) {
			printf("%s has crashed\n", w->name);
			gb->num_crashed_worms++;
			if (gb->num_crashed_worms == gb->num_worms - 1) {
				gb->winner = num_worm;
				gb->game_over = 1;
			} else if (gb->num_crashed_worms >= gb->num_worms) {
				gb->winner = DRAW;
				gb->game_over = 1;
			}
		}
	}
}

static worm_crash detect_crash(gameboard_cell cell) {
	if (0 <= cell && cell < MAX_WORMS)
		return WORM_WORM_CRASH;
	else if (cell == CELL_WALL)
		return WORM_WALL_CRASH;
	return WORM_NO_CRASH;
}

void set_gameboard_cell(gameboard* gb, unsigned int x, unsigned int y,
		gameboard_cell value){

	if (x >= gb->width || y >= gb->width) {
		fprintf(stderr, "not setting gameboard cell (%i,%i), it "
				"doesn't exist\n", x, y);
		return;
	}
	gb->grid[gb->width * y + x] = value;
}

gameboard_cell get_gameboard_cell(gameboard *gb, unsigned int x,
		unsigned int y) {

	if (x >= gb->width || y >= gb->width) {
		fprintf(stderr, "cannot get gameboard cell (%i,%i), it "
				"doesn't exist\n", x, y);
		exit(1);
	}
	return gb->grid[gb->width * y + x];
}

