/*
 * Copyright (c) 2005
 *	Stefan Sperling <stsp@binarchy.net>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Stefan Sperling AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Stefan Sperling OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <SDL.h>

#include "wurmer.h"
#include "graphics.h"
#include "graphics_mode.h"
#include "gameboard.h"
#include "worm.h"
#include "menu.h"

static SDL_Surface* new_menu_surface(int width, int height);

int get_graphics_mode_width(graphics_mode gm) {
	switch (gm) {
		case GRAPHICS_MODE_640x480:
			return 640;
		case GRAPHICS_MODE_800x600:
			return 800;
		case GRAPHICS_MODE_1024x768:
			return 1024;
		default:
			fprintf(stderr, "Unknown graphics mode %i\n", gm);
			return -1;
	}
}

int get_graphics_mode_height(graphics_mode gm) {
	switch (gm) {
		case GRAPHICS_MODE_640x480:
			return 480;
		case GRAPHICS_MODE_800x600:
			return 600;
		case GRAPHICS_MODE_1024x768:
			return 768;
		default:
			fprintf(stderr, "Unknown graphics mode %i\n", gm);
			return -1;
	}
}

graphics_window* new_graphics_window(graphics_mode gm, gameboard *gb) {

	int height, width;

	width = get_graphics_mode_width(gm);
	height = get_graphics_mode_height(gm);
	if (width == -1 || height == -1)
		return NULL;

	graphics_window *gw = (graphics_window*)malloc(sizeof(graphics_window));

	gw->gm = gm;

	if (SDL_Init(SDL_INIT_VIDEO) == -1) {
		fprintf(stderr, "Could not init SDL: %s\n", SDL_GetError());
		goto sdl_init_failed;
	}

	gw->gameboard_surface = SDL_SetVideoMode(width, height, BITDEPTH,
			SDL_HWSURFACE | SDL_DOUBLEBUF);

	if (gw->gameboard_surface == NULL) {
		fprintf(stderr, "Could not set video mode %ix%ix%i: %s\n"
				"Trying again with software surface\n",
				width, height, BITDEPTH, SDL_GetError());
		gw->gameboard_surface = SDL_SetVideoMode(width, height, BITDEPTH,
				SDL_SWSURFACE);
	}
	if (gw->gameboard_surface == NULL) {
		fprintf(stderr, "Could not set video mode %ix%ix%i: %s\n",
				width, height, BITDEPTH, SDL_GetError());
		goto set_video_mode_failed;
	}

	gw->menu_surface = new_menu_surface(width, height);
	if (gw->menu_surface == NULL) {
		fprintf(stderr, "Could not create menu surface %s\n",
				SDL_GetError());
		goto new_menu_surface_failed;
	}
 

	SDL_WM_SetCaption(CAPTION, NULL);

	return gw;

new_menu_surface_failed:
	SDL_FreeSurface(gw->gameboard_surface);
set_video_mode_failed:
	SDL_Quit();
sdl_init_failed:
	free(gw);
	return NULL;
}

static SDL_Surface* new_menu_surface(int width, int height) {

	Uint32 rmask, gmask, bmask, amask;
#if SDL_BYTEORDER == SDL_BIG_ENDIAN
	rmask = 0xff000000;
	gmask = 0x00ff0000;
	bmask = 0x0000ff00;
	amask = 0x000000ff;
#else
	rmask = 0x000000ff;
	gmask = 0x0000ff00;
	bmask = 0x00ff0000;
	amask = 0xff000000;
#endif

	/* mmmh... do we really need to create a temporary surface just
	 * to create a copy of it that heeds the display format???
	 * someone please help out */

	SDL_Surface *s = SDL_CreateRGBSurface(SDL_HWSURFACE, width, height,
			BITDEPTH, rmask, gmask, bmask, amask);
	if (s == NULL) {
		fprintf(stderr, "Could not create new RGB surface: %s\n",
				SDL_GetError());
		return NULL;
	}

	SDL_SetAlpha(s, SDL_SRCALPHA | SDL_RLEACCEL, MENU_ALPHA);

	SDL_Surface *out = SDL_DisplayFormat(s);
	if (out == NULL) {
		fprintf(stderr, "Could not convert temporary surface to "
				"display format: %s\n", SDL_GetError());
		return NULL;
	}

	SDL_FreeSurface(s);

	return out;
}


void kill_graphics_window(graphics_window *gw) {
	SDL_FreeSurface(gw->menu_surface);
	SDL_FreeSurface(gw->gameboard_surface);
	free(gw);
	gw = NULL;
	SDL_Quit();
}

void draw(wurmer *w) {
	unsigned int i, j, k, n;
	unsigned int pixel_rows_per_cell_row, cell_rows, cell_cols;
	int width, height;
	Uint16 *pixel_row, *pixel;
	gameboard_cell cell;

	/* Generate color lookup table for drawing loop below */
	Uint16 colors[NUM_CELL_STATES];
	for (i = 0; i < NUM_CELL_STATES; i++)
		colors[i] = SDL_MapRGB(w->gw->gameboard_surface->format,
				w->gb->colors[i].r, w->gb->colors[i].g,
				w->gb->colors[i].b);

	width = get_graphics_mode_width(w->gw->gm);
	height = get_graphics_mode_height(w->gw->gm);
	if (width == -1 || height == -1)
		return;

	SDL_LockSurface(w->gw->gameboard_surface);

	pixel_row = (Uint16*)w->gw->gameboard_surface->pixels; 
	cell_rows = height / w->gb->cell_size;
	pixel_rows_per_cell_row = height / cell_rows;
	cell_cols = width / w->gb->cell_size; 

	/* action */
	for (i = 0; i < cell_rows; i++) {
		for (j = 0; j < pixel_rows_per_cell_row; j++) {
			pixel = pixel_row; 
			for (k = 0; k < cell_cols; k ++) {
				cell = get_gameboard_cell(w->gb, k, i);
				for (n = 0; n < w->gb->cell_size; n++)
					*(pixel + n) = colors[cell];
				pixel += w->gb->cell_size; 
			}
			pixel_row += w->gw->gameboard_surface->pitch / 2; 
		}
	}

	SDL_UnlockSurface(w->gw->gameboard_surface);

	/* blit the menu surface onto the gameboard surface if
	 * in menu */
	if (w->game_state == GAME_MENU) {
		if(SDL_BlitSurface(w->gw->menu_surface, NULL,
					w->gw->gameboard_surface, NULL) == -1 ) 
			fprintf(stderr, "Could not blit menu surface: %s\n",
					SDL_GetError());
	}

	SDL_Flip(w->gw->gameboard_surface);
}
