/*
 * Copyright (c) 2005
 *	Stefan Sperling <stsp@binarchy.net>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Stefan Sperling AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Stefan Sperling OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <SDL.h>
#include <SDL_ttf.h>

#include "menu.h"
#include "graphics.h"
#include "wurmer.h"

static const char *entry_texts[NUM_MENU_ENTRIES] = {
	/* Some of these are dynamic. For these, the values given here
	 * are only dummies. */
	"New Game", /* ENTRY_NEW_GAME */
	"Players: ", /* ENTRY_PLAYERS */
	"Resolution: ", /* ENTRY_RESOLUTION */
	"Quit" /* ENTRY_QUIT_GAME */
		
};

static menu_entry* new_menu_entry(menu *m, const char *text, int x, int y);
static void kill_menu_entry(menu_entry *e);

menu* new_menu(graphics_window *gw) {

	menu *m = (menu*)malloc(sizeof(menu));
	if (m == NULL) {
		fprintf(stderr, "Could not allocate memory for menu\n");
		return NULL;
	}

	if (TTF_Init() == -1) {
		fprintf(stderr, "Could not initialize SDL_TTF\n");
		return NULL;
	}

	m->title_font = TTF_OpenFont(MENU_FONT_FILE, MENU_TITLE_FONT_SIZE);
	m->entry_font = TTF_OpenFont(MENU_FONT_FILE, MENU_ENTRY_FONT_SIZE);
	m->font_color.r = 0xff;
	m->font_color.g = 0xff;
	m->font_color.b = 0x00;

	m->shaded_font_color.r = 0x99;
	m->shaded_font_color.g = 0x99;
	m->shaded_font_color.b = 0x33;

	m->title = TTF_RenderText_Blended(m->title_font, MENU_TITLE,
			m->font_color);
	if(m->title == NULL)
		fprintf(stderr, "Could not render menu title: %s\n",
				TTF_GetError());
	
	unsigned int width = get_graphics_mode_width(gw->gm);
	unsigned int height = get_graphics_mode_height(gw->gm);
	m->title_position.x = (width / 2) - (m->title->w / 2);
	m->title_position.y = height / 6;
	m->title_position.w = m->title->w;
	m->title_position.h = m->title->h;

	/* create menu entries */
	unsigned int i, y, spacing;
	spacing = TTF_FontLineSkip(m->entry_font);
	y = height / 3;

	for (i = 0; i < NUM_MENU_ENTRIES; i++) {
		m->entries[i] = new_menu_entry(m, entry_texts[i], 0, y);
		m->entries[i]->position.x = width / 2
			- (m->entries[i]->text->w / 2);
		y += spacing;
	}

	m->active_entry = 0;

	return m;
}

void kill_menu(menu *m) {
	unsigned int i;
	for (i = 0; i < NUM_MENU_ENTRIES; i++) {
		kill_menu_entry(m->entries[i]);
	}
	SDL_FreeSurface(m->title);
	TTF_CloseFont(m->title_font);
	free(m);
	m = NULL;
	TTF_Quit();
}

void draw_menu(wurmer *w) {

	if (SDL_FillRect(w->gw->menu_surface, NULL,
				SDL_MapRGB(w->gw->menu_surface->format,
					0, 0, 0)) == -1) {
		fprintf(stderr, "Could not clear menu surface: %s\n",
				SDL_GetError());
	}
	
	if(SDL_BlitSurface(w->m->title, NULL, w->gw->menu_surface,
				&w->m->title_position)
			== -1) 
		fprintf(stderr, "Could not blit menu surface: %s\n",
				SDL_GetError());

	unsigned int i;
	SDL_Surface *t = NULL;
	char players_text[11];
	char graphics_mode_text[21];

	for (i = 0; i < NUM_MENU_ENTRIES; i++) {

		if (i == ENTRY_PLAYERS) {
			snprintf(players_text, 11, "Players: %i",
					w->options->num_worms);
			int x = w->m->entries[i]->position.x;
			int y = w->m->entries[i]->position.y;
			kill_menu_entry(w->m->entries[i]);
			w->m->entries[i] = new_menu_entry(w->m, players_text,
					x, y);
		}

		else if (i == ENTRY_RESOLUTION) {
			snprintf(graphics_mode_text, 21, "Resolution: %ix%i",
					get_graphics_mode_width(w->gw->gm),
					get_graphics_mode_height(w->gw->gm));
			int x = w->m->entries[i]->position.x;
			int y = w->m->entries[i]->position.y;
			kill_menu_entry(w->m->entries[i]);
			w->m->entries[i] = new_menu_entry(w->m,
					graphics_mode_text, x, y);
		}

		if (w->m->active_entry == i)
			t = w->m->entries[i]->text;
		else
			t = w->m->entries[i]->shaded_text;

		if (SDL_BlitSurface(t, NULL, w->gw->menu_surface,
					&w->m->entries[i]->position)
				== -1)
			fprintf(stderr,	"Could not blit menu entry surface: "
					"%s\n", SDL_GetError());
	}
}

static menu_entry* new_menu_entry(menu *m, const char *text, int x, int y) {

	menu_entry *e = (menu_entry*)malloc(sizeof(menu_entry));
	e->text = TTF_RenderText_Blended(m->entry_font, text, m->font_color);
	if(e->text == NULL) {
		fprintf(stderr, "Could not render menu entry text: %s\n",
				TTF_GetError());
		goto rendering_text_failed;
	}
	e->shaded_text = TTF_RenderText_Blended(m->entry_font, text,
			m->shaded_font_color);
	if(e->shaded_text == NULL) {
		fprintf(stderr, "Could not render menu entry text: %s\n",
				TTF_GetError());
		goto rendering_shaded_text_failed;
	}
	
	e->position.x = x;
	e->position.y = y;
	e->position.w = e->text->w;
	e->position.h = e->text->h;

	return e;

rendering_shaded_text_failed:
	SDL_FreeSurface(e->text);
rendering_text_failed:
	free(e);
	return NULL;
}

static void kill_menu_entry(menu_entry *e) {
	SDL_FreeSurface(e->text);	
	SDL_FreeSurface(e->shaded_text);
	free(e);
	e = NULL;
}
