/*
 * Copyright (c) 2005
 *	Stefan Sperling <stsp@binarchy.net>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Stefan Sperling AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Stefan Sperling OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include <SDL.h>

#include "wurmer.h"
#include "worm.h"
#include "gameboard.h"
#include "graphics.h"
#include "event.h"
#include "options.h"
#include "menu.h"

wurmer* new_wurmer(wurmer_options *o) {

	wurmer *w = (wurmer*)malloc(sizeof(wurmer));
	if (w == NULL) {
		fprintf(stderr, "Could not allocate memory for wurmer\n");
		return NULL;
	}

	w->options = o;

	/* gameboard */
	w->gb = new_gameboard(o->gm, o->num_worms);
	if (w->gb == NULL) {
		fprintf(stderr, "Could not create gameboard\n");
		goto new_gameboard_failed;
	}

	/* graphics_window */
	w->gw = new_graphics_window(o->gm, w->gb);
	if (w->gw == NULL) {
		fprintf(stderr, "Could not create graphics window\n");
		goto new_graphics_window_failed;
	}

	w->delay = DEFAULT_DELAY;

#if 0
	printf("generating level...\n");
	generate_level(w->gb, l);
#endif

	w->m = new_menu(w->gw);
	if (w->m == NULL) {
		fprintf(stderr, "Could not create menu\n");
		goto new_menu_failed;
	}

	w->game_state = GAME_MENU;

	return w;

new_menu_failed:
	kill_graphics_window(w->gw);
new_graphics_window_failed:
	kill_gameboard(w->gb);
new_gameboard_failed:
	free(w);
	return NULL;
}

void kill_wurmer(wurmer *w) {
	/* don't free w->options !!! */
	w->options = NULL;
	kill_menu(w->m);
	w->m = NULL;
	kill_graphics_window(w->gw);
	w->gw = NULL;
	kill_gameboard(w->gb);
	w->gb = NULL;
	free(w);
	w = NULL;
}

wurmer* reset_wurmer(wurmer *w) {
	wurmer_options *o = w->options;
	kill_wurmer(w);
	wurmer *nw = new_wurmer(o);
	if (nw == NULL)
		fprintf(stderr, "Could not reset game\n");
	return nw;
}

void reset_gameboard(wurmer *w) {
	kill_gameboard(w->gb);
	w->gb = new_gameboard(w->options->gm, w->options->num_worms);
	if (w->gb == NULL)
		fprintf(stderr, "Could not reset gameboard\n");
}
/* returns either GAME_NEW or GAME_QUIT */
int run_game(wurmer *w) {
	Uint32 ticks, now;

	w->game_state = GAME_MENU;

	ticks = SDL_GetTicks();

	while (w->game_state != GAME_QUIT) {
		now = SDL_GetTicks();
		if (now - ticks > w->delay) {
			w->game_state = get_input(w);
			switch (w->game_state) {
				case GAME_MENU:
					draw_menu(w);
					break;
				case GAME_RUNNING:
					move_worms(w->gb);
					if (w->gb->game_over) {
						w->game_state = GAME_MENU;
					}
					break;
				default:
					return w->game_state;
			}
			draw(w);
			ticks = now;
		}

	}

	printf("%s was the longest surviver\n",
			w->gb->worms[w->gb->winner]->name);
	return GAME_QUIT;
}

